const express = require('express');
const admin = require('firebase-admin');
const WebSocket = require('ws');
const url = require('url');
const querystring = require('querystring');
const client = express();
const port = process.env.PORT || 8000; // Change the port as needed

const { initializeApp, applicationDefault, cert } = require('firebase-admin/app');
const { getFirestore, Timestamp, FieldValue, Filter } = require('firebase-admin/firestore');

const serviceAccount = require('./assets/oreba.json');

initializeApp({
  credential: cert(serviceAccount)
});

const db = getFirestore();

// Create an HTTP server using Express
const server = client.listen(port, () => {
    console.log(`Server is running on http://localhost:${port}`);
});

// Create a WebSocket server using the ws library
const wss = new WebSocket.Server({ server });


// Enable Cross-Origin Resource Sharing (CORS) to allow WebSocket connections from any origin
wss.on('connection', (ws, req) => {
    // Handle WebSocket connections here
    // Extract the query parameters from the WebSocket URL
    const query = url.parse(req.url).query;
    const queryParams = querystring.parse(query);
    // Access the parameter
    const collection = queryParams.collection;
    if (queryParams.documentId !== undefined) {
        const documentId = queryParams.documentId;
        var observerRef = db.collection(collection).doc(documentId);
        //DOCUMENT--------------------------------------------
        var map = {};
        var observer = observerRef.onSnapshot((snapshot) => {
            var map = snapshot.data();
            map['id'] = snapshot.id;
            // Send the data to all connected WebSocket clients
            wss.clients.forEach((client) => {
                if (client.readyState === WebSocket.OPEN) {
                    client.send(JSON.stringify(map));
                }
            });
        }, (err) => {
            console.error(`Error: ${err}`);
        });
        //DOCUMENT--------------------------------------------
    } else {
        var observerRef = db.collection(collection);
        //COLLECTION--------------------------------------------
        var observer = observerRef.onSnapshot((snapshot) => {
            snapshot.docChanges().forEach((change) => {
                var map = { collection: collection, type: change.type.toUpperCase(), id: change.doc.id, data: change.doc.data() }
                // Send the data to all connected WebSocket clients
                wss.clients.forEach((client) => {
                    if (client.readyState === WebSocket.OPEN) {
                        client.send(JSON.stringify(map));
                    }
                });
            });
        }, (err) => {
            console.error(`Error: ${err}`);
        });
        //COLLECTION--------------------------------------------
    }

    ws.on('message', (message) => {
        // Handle incoming messages from the WebSocket client (if needed)
    });

    ws.on('close', (code, reason) => {
        observer();
    });
});